@@ -2,13 +2,17 @@ package OpenFrame::AppKit;
 
 use OpenFrame::AppKit::App;
 use OpenFrame::AppKit::Session;
-use OpenFrame::AppKit::Segment::TT2;
+use OpenFrame::AppKit::Segment::Images;
+use OpenFrame::AppKit::Segment::LogFile;
 use OpenFrame::AppKit::Segment::SessionLoader;
+use OpenFrame::AppKit::Segment::TT2;
 
-our $VERSION = '1.01';
+our $VERSION = '1.03';
 
 1;
 
+__END__
+
 =head1 NAME
 
 OpenFrame::AppKit - The OpenFrame AppKit
@@ -36,14 +40,22 @@ Base application class for OpenFrame::AppKit
 
 Sessions for OpenFrame::AppKit
 
-=item * OpenFrame::AppKit::Segment::TT2
+=item * OpenFrame::AppKit::Segment::Images
 
-A Template Toolkit template engine for OpenFrame::AppKit
+Static images handler for OpenFrame::AppKit
+
+=item * OpenFrame::AppKit::Segment::LogFile
+
+Simple logger for OpenFrame::AppKit
 
 =item * OpenFrame::AppKit::Segment::SessionLoader
 
 A session management segment for OpenFrame::AppKit
 
+=item * OpenFrame::AppKit::Segment::TT2
+
+A Template Toolkit template engine for OpenFrame::AppKit
+
 =back
 
 OpenFrame::AppKit also has some example applications in the form of
